///////////////////////////////////////////////////////////

function Combatant(name, filename) {
  this.name = name;
  this.type = "monster";
  this.minhp = 0;
  this.hp = 30;
  this.maxhp = 45;
  this.str = 10;
  this.status = "normal";

  // all this is to do with the drawing
  this.spriteset = LoadSpriteset(filename);
  this.x = 50;
  this.y = 75;
  this.width = 16;
  this.height = 32;
  this.adjuster_x = 0;
  this.adjuster_y = 0;
  this.last_update = 0;
  this.frame_index = 0;
  this.direction_index = 0;
}

Combatant.prototype.clone = function() {
  var tmp = new Object();
  for (var i in this)
    tmp[i] = this[i];
  return tmp;
}

Combatant.prototype.AI = function() {
  var list_of_who_to_attack = new Array();
  for (var i = 0; i < gCombatants.length; ++i)
    if (gCombatants[i].type != this.type)
      list_of_who_to_attack.push(i);
  var who_to_attack = Math.floor(Math.random() * list_of_who_to_attack.length);
  Attack(gTurnlist[0], list_of_who_to_attack[who_to_attack]);
}

///////////////////////////////////////////////////////////

function Item(name) {
  this.name = name;
  this.power = -20;
  this.amount = 20;
}

Item.prototype.on_battle_use = function(from_who, on_who) {
  var damage = this.power;
  gCombatants[on_who].hp -= damage;
  gDamageAnimations.push(CreateDamageAnimationObject(on_who, damage));
  --this.amount;
}

///////////////////////////////////////////////////////////

function Spell(name) {
  this.name = name;
  this.power = -20;
  this.cost = 35;
}

Spell.prototype.on_battle_use = function(from_who, on_who) {
  var damage = this.power;
  gCombatants[on_who].hp -= damage;
  gDamageAnimations.push(CreateDamageAnimationObject(on_who, damage));
  --this.amount;
}

///////////////////////////////////////////////////////////

function Attack(attacker, attackee) {
  var damage = gCombatants[attacker].str;
  gCombatants[attackee].hp -= damage;
  gDamageAnimations.push(CreateDamageAnimationObject(attackee, damage));
}

///////////////////////////////////////////////////////////

function AttackMenu() {
  var menu = DefaultMenu(96, 32);
  var combatant_list = GetAliveMonstersIndexArray().concat(GetAlivePlayersIndexArray());

  for (var i = 0; i < gCombatants.length; ++i)
    menu.addText(gCombatants[combatant_list[i]].name, function() {  Attack(gTurnlist[0], combatant_list[menu.selection]); menu.done = true; });

  menu.postRender = function() {
    var combatant_index = combatant_list[menu.selection];
    var pointer = menu.items[menu.selection].item.pointer;
    pointer.blit(gCombatants[combatant_index].x - pointer.width, gCombatants[combatant_index].y + (gCombatants[combatant_index].height / 2 - pointer.height / 2));
  }

  gBattleMenuArray.push(menu);
}

///////////////////////////////////////////////////////////

function ItemWhoMenu(which_item) {
  var menu = DefaultMenu(96, 32);
  var combatant_list = GetAlivePlayersIndexArray().concat(GetAliveMonstersIndexArray());

  for (var i = 0; i < gCombatants.length; ++i)
    menu.addText(gCombatants[combatant_list[i]].name, function() {  gItemList[which_item].on_battle_use(gTurnlist[0], combatant_list[menu.selection]); menu.done = true; });

  menu.postRender = function() {
    var combatant_index = combatant_list[menu.selection];
    var pointer = menu.items[menu.selection].item.pointer;
    pointer.blit(gCombatants[combatant_index].x - pointer.width, gCombatants[combatant_index].y + (gCombatants[combatant_index].height / 2 - pointer.height / 2));
  }

  gBattleMenuArray.push(menu);
}

///////////////////////////////////////////////////////////

function ItemMenu() {
  var menu = DefaultMenu(150, 32);
  var item_list = [0, 1];

  for (var i = 0; i < item_list.length; ++i)
    menu.addText(gItemList[item_list[i]].name, function() { ItemWhoMenu(item_list[menu.selection]); });

  menu.postRender = function() {
    for (var i = 0; i < item_list.length; ++i)
      GetSystemFont().drawText(menu.x + menu.width - GetSystemFont().getStringWidth(gItemList[item_list[i]].amount), menu.items[i].y, gItemList[item_list[i]].amount);
  }

  gBattleMenuArray.push(menu);
}

///////////////////////////////////////////////////////////

function SpellWhoMenu(which_spell) {
  var menu = DefaultMenu(96, 32);
  var combatant_list = GetAlivePlayersIndexArray().concat(GetAliveMonstersIndexArray());

  for (var i = 0; i < gCombatants.length; ++i)
    menu.addText(gCombatants[combatant_list[i]].name, function() {  gSpellList[which_spell].on_battle_use(gTurnlist[0], combatant_list[menu.selection]); menu.done = true; });

  menu.postRender = function() {
    var combatant_index = combatant_list[menu.selection];
    var pointer = menu.items[menu.selection].item.pointer;
    pointer.blit(gCombatants[combatant_index].x - pointer.width, gCombatants[combatant_index].y + (gCombatants[combatant_index].height / 2 - pointer.height / 2));
  }

  gBattleMenuArray.push(menu);
}

///////////////////////////////////////////////////////////

function SpellMenu() {
  var menu = DefaultMenu(150, 32);
  var spell_list = [0];

  for (var i = 0; i < spell_list.length; ++i)
    menu.addText(gSpellList[spell_list[i]].name, function() { SpellWhoMenu(spell_list[menu.selection]); });

  menu.postRender = function() {
    for (var i = 0; i < spell_list.length; ++i)
      GetSystemFont().drawText(menu.x + menu.width - GetSystemFont().getStringWidth(gSpellList[spell_list[i]].cost), menu.items[i].y, gSpellList[spell_list[i]].amount);
  }

  gBattleMenuArray.push(menu);
}

///////////////////////////////////////////////////////////

Combatant.prototype.generate_menu = function() {
  var menu = DefaultMenu(16, 16);
  menu.addText("Attack", AttackMenu);
  menu.addText("Spells", SpellMenu);
  menu.addText("Items", ItemMenu);
  menu.addText("Escape", function() { AttemptEscape(); menu.done = true; });
  return menu;
}

///////////////////////////////////////////////////////////

Combatant.prototype.deal_with_turn = function() {
  // called whenever it is this person's turn
  gCombatants[gTurnlist[0]].turn_start_time = GetTime();

  if (!this.is_defeated())
    if (this.type == "player" && this.status != "berserk")
      gBattleMenuArray.push(this.generate_menu());
    else
    {
      gBattleMenuArray = new Array();
      this.AI();
      TurnFinished();
    }
  else
    TurnFinished();
}

///////////////////////////////////////////////////////////

Combatant.prototype.is_defeated = function() {
  return this.hp <= 0;
}

Combatant.prototype.get_total_health = function() {
  return (this.maxhp - this.minhp);
}

///////////////////////////////////////////////////////////

Combatant.prototype.update = function() {
  if (this.last_update + 80 < GetTime())
    this.frame_index += 1;

  if (this.frame_index >= this.spriteset.directions[this.direction_index].frames.length)
    this.frame_index = 0;
}

Combatant.prototype.draw = function() {
  var x = this.x + this.adjuster_x;
  var y = this.y + this.adjuster_y;
  this.spriteset.images[this.spriteset.directions[this.direction_index].frames[this.frame_index].index].blit(x, y);
  if (this.type == "player")
    GetSystemFont().drawText(x, y + this.height, this.hp + " / " + this.get_total_health());
}

/////////////////////////////////

var gCombatants = new Array();
var gBattleDone = true;
var gTurnlist   = new Array();
var gBattleMenuArray = new Array();
var gBattleTextArray = new Array();
var gDamageAnimations = new Array();

var gItemList = [new Item("Potion"), new Item("Herb")];
var gSpellList = [new Item("Cure")];

function RenderBattle() {
  for (var i = 0; i < gCombatants.length; ++i) {
    gCombatants[i].update();
    gCombatants[i].draw();
  }
}

function BattleText(text) {
  gBattleTextArray.push(text);
}

function AttemptEscape() {
  BattleText(gTurnlist[0].name + " tries to escape...");
  if (gCombatants[gTurnlist[0]].type == "player")
    BattleText("The party escapes.");

  gBattleDone = true;
}

function DealTurns() {
  for (var i = 0; i < gCombatants.length; ++i)
    gTurnlist.push(i);
}

///////////////////////////////////////////////////////////

function GetMonstersIndexArray(){
  var indexes = new Array();
  for (var i = 0; i < gCombatants.length; ++i)
    if(gCombatants[i].type == "monster")
      indexes.push(i);
  return indexes;
}

function TotalMonsters() {
  return GetMonstersIndexArray().length;
}

function GetPlayersIndexArray() {
  var indexes = new Array();
  for (var i = 0; i < gCombatants.length; ++i)
    if(gCombatants[i].type == "player")
      indexes.push(i);
  return indexes;
}

function TotalPlayers() {
  return GetPlayersIndexArray().length;
}

function GetAlivePlayersIndexArray() {
  var indexes = new Array();
  for (var i = 0; i < gCombatants.length; ++i)
    if(gCombatants[i].type == "player" && !gCombatants[i].is_defeated())
      indexes.push(i);
  return indexes;
}

function TotalAlivePlayers() {
  return GetAlivePlayersIndexArray().length;
}

function GetDeadPlayersIndexArray() {
  var indexes = new Array();
  for (var i = 0; i < gCombatants.length; ++i)
    if(gCombatants[i].type == "player" && gCombatants[i].is_defeated())
      indexes.push(i);
  return indexes;
}

function TotalDeadPlayers() {
  return GetDeadPlayersIndexArray().length;
}

function GetAliveMonstersIndexArray() {
  var indexes = new Array();
  for (var i = 0; i < gCombatants.length; ++i)
    if(gCombatants[i].type == "monster" && !gCombatants[i].is_defeated())
      indexes.push(i);
  return indexes;
}

function TotalAliveMonsters() {
  return GetAliveMonstersIndexArray().length;
}

function GetDeadMonstersIndexArray() {
  var indexes = new Array();
  for (var i = 0; i < gCombatants.length; ++i)
    if(gCombatants[i].type == "monster" && gCombatants[i].is_defeated())
      indexes.push(i);
  return indexes;
}

function TotalDeadMonsters() {
  return GetDeadMonstersIndexArray().length;
}

function RemoveCombatants(indexes) {
  var tmp = new Array();
  for (var i = 0; i < gCombatants.length; ++i) {
    var remove = false;
    for (var j = 0; j < indexes.length; ++j)
      if (indexes[j] == i)
        remove = true;
    if (!remove)
      tmp.push(gCombatants[i].clone());
  }
  gCombatants = tmp;
}

///////////////////////////////////////////////////////////

function HasBattleFinished() {
  if (TotalMonsters() == TotalDeadMonsters())  
    gBattleDone = true;

  if (TotalPlayers() == TotalDeadPlayers())
    gBattleDone = true;

  return (gBattleDone);
}

function CloneParty() {
  var clones = new Array();
  for (var i = 0; i < gCombatants.length; ++i)
    if (gCombatants[i].type == "player")
      clones.push(gCombatants[i].clone());
  return clones;
}

function CloneCombatants() {
  var clones = new Array();
  for (var i = 0; i < gCombatants.length; ++i)
    clones.push(gCombatants[i].clone());
  return clones;
}

function TurnFinished() {
  gTurnlist = gTurnlist.slice(1);
}

///////////////////////////////////////////////////////////
function CreateWolfy()
{
  var wofly = new Combatant("Wolfy", "wolfy.rss");
  wofly.type = "monster";
  gCombatants.push(wofly.clone());
}

function CreateJimmy() {
  var jimmy = new Combatant("Jimmy", "jimmy.rss");
  jimmy.type = "player";
  jimmy.x = 200;
  gCombatants.push(jimmy.clone());
}

function CreateSkelton()
{
  var skelton = new Combatant("Bones", "skeleton.rss");
  skelton.type = "monster";
  gCombatants.push(skelton.clone());
}
  
function BattleSkelton() {
  CreateSkelton();
  if (battle() == true) {
    DestroyPerson(GetCurrentPerson());
  }
}

function BattleWolfy() {
  CreateSkelton();
  if (battle() == true) {
    DestroyPerson(GetCurrentPerson());
  }
}
  
/////////////////////////////////////

function Debug(text) {
  GetSystemFont().drawText(16, 16, text);
  FlipScreen();
  Wait();
}

///////////////////////////////////////////////////////////

function battle() {
  var can_lose = true;
  var can_run_away = true;

  gBattleDone = false;
  gBattleMenuArray = new Array();

  var background = LoadImage("battle beach.png");

  while (!HasBattleFinished()) {
    background.blit(0,0);
    RenderBattle();

    if (gTurnlist.length == 0)
      DealTurns();

    if (gTurnlist.length > 0 && gBattleMenuArray.length == 0) {
      gCombatants[gTurnlist[0]].deal_with_turn();
    }

    if (gBattleMenuArray.length > 0) {
      for (var i = 0; i < gBattleMenuArray.length; ++i) {
        gBattleMenuArray[i].has_key_focus = (i == gBattleMenuArray.length - 1);
        gBattleMenuArray[i].has_mouse_focus = (i == gBattleMenuArray.length - 1);

        gBattleMenuArray[i].preRender();
        gBattleMenuArray[i].go();
        gBattleMenuArray[i].postRender();
      }

      if (gBattleMenuArray[gBattleMenuArray.length - 1].done) {
        gBattleMenuArray = new Array();
        TurnFinished();
      }
    }

    if (gBattleTextArray.length > 0) {
      var background = GrabImage(0, 0, GetScreenWidth(), GetScreenHeight());
      GetSystemFont().drawTextBox(16, GetScreenHeight() * 3 / 4, GetScreenWidth() - 32,
                                  GetScreenHeight() / 4, 0, gBattleTextArray[0]);
      if (IsKeyPressed(KEY_SPACE))
        gBattleTextArray = gBattleTextArray.slice(1);
    }

    for (var i = 0; i < gDamageAnimations.length; ++i)
    {
      gDamageAnimations[i].update();
      gDamageAnimations[i].render();
    }

    FlipScreen();
    
  }

  RemoveCombatants(GetMonstersIndexArray());
  gBattleTextArray = new Array();
  gDamageAnimations = new Array();
  
  return (TotalDeadMonsters() == 0);
}

///////////////////////////////////////////////////////////

function DamageAnimation(startx, starty, endx, endy, start_color, end_color, time, text) {
  this.startx = startx;
  this.starty = starty;
  this.endx = endx;
  this.endy = endy;
  this.start_color = start_color;
  this.end_color = end_color;
  this.time = time;
  this.start_time = GetTime();
  this.text = text;

  this.x = startx;
  this.y = starty;
  this.color = start_color;

  this.dx = this.endx - this.startx;
  this.dy = this.endy - this.starty;
}

DamageAnimation.prototype.update = function() {
  var time_gone = GetTime() - this.start_time;
  var time_left = this.time - time_gone;
  this.color = BlendColorsWeighted(this.start_color, this.end_color, time_left, time_gone);

  this.x = this.startx + (time_gone * this.dx / this.time);
  this.y = this.starty + (time_gone * this.dy / this.time);
}

DamageAnimation.prototype.render = function() {
  if (this.time + this.start_time > GetTime())
  {
    var font = GetSystemFont();
    font.setColorMask(this.color);
    font.drawText(this.x, this.y, this.text);
  }
}

///////////////////////////////////////////////////////////

function CreateDamageAnimationObject(who, how_much_damage)
{
  var startx = gCombatants[who].x + (gCombatants[who].width / 2);
  var starty = gCombatants[who].y + (gCombatants[who].height / 2);
  var endx = startx;
  var endy = starty - 20;
  var time = 500;
  var text = how_much_damage >= 0 || how_much_damage <= 0 ? Math.abs(how_much_damage) : how_much_damage;

  var start_color = CreateColor(255, 255, 255);

  // end color
  var end_color = CreateColor(255, 255, 255);
  if (how_much_damage > 0) {
    end_color = CreateColor(255, 0, 0);
  }
  else if (how_much_damage < 0) {
    end_color = CreateColor(0, 255, 0);
  }

  return new DamageAnimation(startx, starty, endx, endy, start_color, end_color, time, text)
}

///////////////////////////////////////////////////////////
